<?php

declare(strict_types=1);

namespace App\Dictionary\Patient;

/**
 * 证件类型，来自国家医保字典
 */
enum IdentifyCardType: string
{
    case RESIDENT_ID_CARD = '01';
    case CHINESE_PEOPLES_LIBERATION_ARMY_OFFICER_CARD = '02';
    case ARMED_POLICE_OFFICER_CARD = '03';
    case HK_SAR_PASSPORT_OR_MAINLAND_TRAVEL_PERMIT = '04';
    case MACAU_SAR_PASSPORT_OR_MAINLAND_TRAVEL_PERMIT = '05';
    case TAIWAN_TRAVEL_PERMIT = '06';
    case FOREIGNER_PERMANENT_RESIDENCE_PERMIT = '07';
    case FOREIGNER_PASSPORT = '08';
    case DISABLED_PERSON_CARD = '09';
    case MARTYR_OR_FAMILY_MEMBER_PROOF = '10';
    case FOREIGNER_EMPLOYMENT_PERMIT = '11';
    case FOREIGN_EXPERT_CERTIFICATE = '12';
    case FOREIGN_PERMANENT_REPORTER_CERTIFICATE = '13';
    case EMPLOYMENT_PERMIT_FOR_TAIWAN_HK_MACAU_RESIDENTS = '14';
    case RETURNED_EXPERT_CERTIFICATE = '15';
    case SOCIAL_SECURITY_CARD = '90';
    case OTHER_IDENTITY_DOCUMENT = '99';

    public function label(): string
    {
        return match ($this) {
            self::RESIDENT_ID_CARD => '居民身份证（户口簿）',
            self::CHINESE_PEOPLES_LIBERATION_ARMY_OFFICER_CARD => '中国人民解放军军官证',
            self::ARMED_POLICE_OFFICER_CARD => '中国人民武装警察警官证',
            self::HK_SAR_PASSPORT_OR_MAINLAND_TRAVEL_PERMIT => '香港特区护照/港澳居民来往内地通行证',
            self::MACAU_SAR_PASSPORT_OR_MAINLAND_TRAVEL_PERMIT => '澳门特区护照/港澳居民来往内地通行证',
            self::TAIWAN_TRAVEL_PERMIT => '台湾居民来往大陆通行证',
            self::FOREIGNER_PERMANENT_RESIDENCE_PERMIT => '外国人永久居留证',
            self::FOREIGNER_PASSPORT => '外国人护照',
            self::DISABLED_PERSON_CARD => '残疾人证',
            self::MARTYR_OR_FAMILY_MEMBER_PROOF => '军烈属证明',
            self::FOREIGNER_EMPLOYMENT_PERMIT => '外国人就业证',
            self::FOREIGN_EXPERT_CERTIFICATE => '外国专家证',
            self::FOREIGN_PERMANENT_REPORTER_CERTIFICATE => '外国人常驻记者证',
            self::EMPLOYMENT_PERMIT_FOR_TAIWAN_HK_MACAU_RESIDENTS => '台港澳人员就业证',
            self::RETURNED_EXPERT_CERTIFICATE => '回国（来华）定居专家证',
            self::SOCIAL_SECURITY_CARD => '社会保障卡',
            self::OTHER_IDENTITY_DOCUMENT => '其他身份证件',
        };
    }

    /**
     * 具体的校验规则
     * @param string $number
     * @return bool
     */
    public function validateNumber(string $number): bool
    {
        return match ($this) {
            self::RESIDENT_ID_CARD => validateIDCard($number),
            // 军官证
            // 规则： 军/兵/士/文/职/广/（其余中文） + "字第" + 4到8位字母或数字 + "号"
            // 样本： 军字第2001988号, 士字第P011816X号
            self::CHINESE_PEOPLES_LIBERATION_ARMY_OFFICER_CARD => (bool) preg_match("/^[\x{4E00}-\x{9FA5}](字第)([0-9a-zA-Z]{4,8})(号?)$/u", $number),
            // 港澳居民来往内地通行证
            // 规则： H/M + 10位数字
            // 样本： H1234567890
            self::HK_SAR_PASSPORT_OR_MAINLAND_TRAVEL_PERMIT,
            self::MACAU_SAR_PASSPORT_OR_MAINLAND_TRAVEL_PERMIT => (bool) preg_match("/^([HM]\d{10}(\(\w{1}\))?)$/", $number),
            // 台湾居民来往大陆通行证
            // 规则： 10位数字
            // 样本： 1234567890
            self::TAIWAN_TRAVEL_PERMIT => (bool) preg_match("/^\d{10}$/", $number),
            // 2017 / 2024 外国人永居证
            self::FOREIGNER_PERMANENT_RESIDENCE_PERMIT => validateIDCard($number) || validate2017ForeignersIDCard($number),
            // 护照
            // 规则： 14/15开头 + 7位数字, G + 8位数字, P + 7位数字, S/D + 7或8位数字,等
            // 样本： 12345678 或 1234567890B
            // 样本： 141234567, G12345678, P1234567
            self::FOREIGNER_PASSPORT => (bool) preg_match("/^([A-z]|[\d]){5,17}$/", $number),
            // 社会保障卡号码
            // 规则：公民身份证号码 / 香港特别行政区代码（HKG）+预留位（0）+港澳居民来往内地通行证号码第2-9位的终身号；澳门特别行政区代码（MAC）+预留位（0）+港澳居民来往内地通行证号码第2-9位的终身号；台湾地区代码（TWN）+预留位（0）+台湾居民来往大陆通行证号码第1-8位的终身号
            self::SOCIAL_SECURITY_CARD => validateIDCard($number) || preg_match("/^(HKG|MAC|TWN)0\d{8}$/", $number),
            // 以下类型不处理,默认为false
            self::DISABLED_PERSON_CARD,
            self::ARMED_POLICE_OFFICER_CARD,
            self::EMPLOYMENT_PERMIT_FOR_TAIWAN_HK_MACAU_RESIDENTS,
            self::RETURNED_EXPERT_CERTIFICATE,
            self::FOREIGN_PERMANENT_REPORTER_CERTIFICATE,
            self::FOREIGN_EXPERT_CERTIFICATE,
            self::MARTYR_OR_FAMILY_MEMBER_PROOF,
            self::FOREIGNER_EMPLOYMENT_PERMIT,
            self::OTHER_IDENTITY_DOCUMENT => false,
        };
    }
}
