<?php

declare(strict_types=1);

namespace UnifyPayment\Cores;

use UnifyPayment\Cores\Contracts\Config as ConfigInterface;
use UnifyPayment\Cores\Support\Arr;

class Config implements ConfigInterface
{

    /**
     * config data
     *
     * @var string[]
     */
    protected array $items = [];

    /**
     * Config Construct
     *
     * @param array $items
     */
    public function __construct(array $items)
    {
        foreach ($items as $key => $value) {
            $this->set($key, $value);
        }
    }

    /**
     *
     * @param string|array $key
     * @param mixed        $default
     *
     * @return mixed
     */
    public function get($key, $default = null)
    {
        if (is_array($key)) {
            return $this->getMany($key);
        }

        return Arr::get($this->items, $key, $default);
    }

    /**
     *
     * @param array $keys
     *
     * @return array
     */
    public function getMany(array $keys): array
    {
        $config = [];

        foreach ($keys as $key => $default) {
            if (is_numeric($key)) {
                [$key, $default] = [$default, null];
            }

            $config[$key] = Arr::get($this->items, $key, $default);
        }

        return $config;
    }

    /**
     *
     * @param string $key
     * @param mixed  $value
     *
     * @return void
     */
    public function set(string $key, $value = null): void
    {
        Arr::set($this->items, $key, $value);
    }

    /**
     *
     * @param string $key
     *
     * @return bool
     */
    public function has(string $key): bool
    {
        return Arr::has($this->items, $key);
    }

    /**
     * @param string $key
     *
     * @return void
     */
    public function forget(string $key): void
    {
        Arr::forget($this->items, $key);
    }

    /**
     * @return array
     */
    public function all(): array
    {
        return $this->items;
    }

    /**
     * build to array
     *
     * @return array
     */
    public function toArray(): array
    {
        return $this->all();
    }

    /**
     * build to json
     *
     * @param int $option
     *
     * @return string
     *
     * @throws \JsonException
     */
    public function toJson(int $option = JSON_UNESCAPED_UNICODE): string
    {
        return json_encode($this->all(), JSON_THROW_ON_ERROR | $option);
    }

    public function __get(string $key)
    {
        return $this->get($key);
    }

    public function __set(string $key, $value)
    {
        $this->set($key, $value);
    }

    public function __isset(string $key): bool
    {
        return $this->has($key);
    }

    public function __unset(string $key)
    {
        $this->forget($key);
    }

    public function offsetExists($offset): bool
    {
        return $this->has((string)$offset);
    }

    public function offsetGet($offset): mixed
    {
        return $this->get((string)$offset);
    }

    public function offsetSet(mixed $offset, $value):void
    {
        $this->set((string)$offset, $value);
    }

    public function offsetUnset($offset): void
    {
        $this->set((string)$offset, null);
    }
}