<?php

namespace UnifyPayment\Cores\Support;

use DateTimeZone;
use Monolog\Formatter\FormatterInterface;
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\HandlerInterface;
use Monolog\Handler\RotatingFileHandler;
use Monolog\Logger as MonoLogger;
use Psr\Log\LoggerInterface;
use UnifyPayment\Cores\Exceptions\InvalidConfigException;

/**
 * @method void emergency($message, array $context = array())
 * @method void alert($message, array $context = array())
 * @method void critical($message, array $context = array())
 * @method void error($message, array $context = array())
 * @method void warning($message, array $context = array())
 * @method void notice($message, array $context = array())
 * @method void info($message, array $context = array())
 * @method void debug($message, array $context = array())
 * @method void log($message, array $context = array())
 */
class Logger
{

    /**
     * @var LoggerInterface|null
     */
    protected ?LoggerInterface $logger = null;

    /**
     * @var array
     */
    protected array $config = [
        'channel' => 'unify',
        'timezone' => 'Asia/Shanghai',
        'date_format' => 'Y-m-d H:i:s',
        'formatter' => LineFormatter::class,
        'formatter_with' => [],
        'handler' => RotatingFileHandler::class,
        'handler_with' => []
    ];

    /**
     * Set Logger
     *
     * @param LoggerInterface $logger
     *
     * @return $this
     */
    public function setLogger(LoggerInterface $logger): Logger
    {
        $this->logger = $logger;

        return $this;
    }

    /**
     * Get Logger
     *
     * @return LoggerInterface
     *
     * @throws InvalidConfigException
     */
    public function getLogger(): LoggerInterface
    {
        if (is_null($this->logger)) {
            $this->logger = $this->createLogger();
        }

        return $this->logger;
    }

    /**
     * Create logger
     *
     * @return MonoLogger
     *
     * @throws InvalidConfigException
     */
    protected function createLogger(): MonoLogger
    {
        return new MonoLogger(
            $this->config['channel'],
            [$this->createHandler()],
            [],
            new DateTimeZone($this->config['timezone'])
        );
    }

    /**
     * Create handler
     *
     * @return HandlerInterface
     *
     * @throws InvalidConfigException
     */
    protected function createHandler(): HandlerInterface
    {
        if (!is_a($this->config['handler'], HandlerInterface::class, true)) {
            throw new InvalidConfigException(
                $this->config['handler'] . ' must be an instance of ' . HandlerInterface::class
            );
        }

        $handler = new $this->config['handler'](... array_values($this->config['handler_with']));
        $handler->setFormatter($this->createFormatter());

        return $handler;
    }

    /**
     * Create formatter
     *
     * @return FormatterInterface
     *
     * @throws InvalidConfigException
     */
    protected function createFormatter(): FormatterInterface
    {
        if (!is_a($this->config['formatter'], FormatterInterface::class, true)) {
            throw new InvalidConfigException(
                $this->config['formatter'] . ' must be an instance of ' . FormatterInterface::class
            );
        }

        return new $this->config['formatter'](... array_values($this->config['formatter_with']));
    }

    /**
     * Call user function
     *
     * @param string $method
     * @param array  $parameters
     *
     * @return mixed
     * @throws InvalidConfigException
     */
    public function __call(string $method, array $parameters): void
    {
        call_user_func_array([$this->getLogger(), $method], $parameters);
    }

    /**
     * Set config
     *
     * @param array $config
     *
     * @return self
     */
    public function setConfig(array $config): self
    {
        $this->config = array_merge($this->config, $config);

        return $this;
    }

    /**
     * Get config
     *
     */
    public function getConfig(): array
    {
        return $this->config;
    }

}